@@ -1,7 +1,5 @@
 from zephyrus_sc2_parser.events.base_event import BaseEvent
 from zephyrus_sc2_parser.game.game_obj import GameObj
-from zephyrus_sc2_parser.gamedata.unit_data import units
-from zephyrus_sc2_parser.gamedata.building_data import buildings
 import copy
 
 
@@ -12,13 +10,16 @@ class ObjectEvent(BaseEvent):
         self.summary_stats = summary_stats
 
     def _get_or_create_game_object(self):
+        units = self.game.gamedata['units']
+        buildings = self.game.gamedata['buildings']
+
         event = self.event
         unit_tag_index = self.event['m_unitTagIndex']
         unit_tag_recycle = self.event['m_unitTagRecycle']
         game_id = self.protocol.unit_tag(unit_tag_index, unit_tag_recycle)
 
         if self.player is False:
-            for p in self.game.players:
+            for p in self.game.players.values():
                 if game_id in p.objects:
                     self.player = p
                     return self.player.objects[game_id]
@@ -47,6 +48,12 @@ class ObjectEvent(BaseEvent):
                 obj['gas_cost']
             )
 
+            if 'energy' in obj:
+                new_game_obj.energy = obj['energy']
+
+            if 'cooldown' in obj:
+                new_game_obj.cooldown = obj['cooldown']
+
             for value in obj['type']:
                 new_game_obj.type.append(value)
 
@@ -87,6 +94,9 @@ class ObjectEvent(BaseEvent):
                 # print()
 
     def parse_event(self):
+        units = self.game.gamedata['units']
+        buildings = self.game.gamedata['buildings']
+
         obj = self._get_or_create_game_object()
         player = self.player
         event = self.event
@@ -99,6 +109,34 @@ class ObjectEvent(BaseEvent):
 
         if self.type == 'NNet.Replay.Tracker.SUnitInitEvent':
             obj.status = 'in_progress'
+            obj.position = {
+                'x': event['m_x'],
+                'y': event['m_y'],
+            }
+
+            if player.warpgate_cooldowns and 'unit' in obj.type:
+                first_cooldown = player.warpgate_cooldowns[0]
+                time_past_cooldown = gameloop - (first_cooldown[0] + first_cooldown[1])
+
+                if time_past_cooldown >= 0:
+                    player.warpgate_cooldowns.pop(0)
+                    player.warpgate_efficiency = (
+                        player.warpgate_efficiency[0] + first_cooldown[1],
+                        player.warpgate_efficiency[1] + time_past_cooldown
+                    )
+
+            # only warped in units generate this event
+            if 'unit' in obj.type and obj.name != 'Archon':
+                player.warpgate_cooldowns.append((gameloop, obj.cooldown))
+                player.warpgate_cooldowns.sort(key=lambda x: x[0] + x[1])
+
+                warpgate_count = 0
+                for obj_id, obj in player.objects.items():
+                    if obj.name == 'WarpGate':
+                        warpgate_count += 1
+
+                while len(player.warpgate_cooldowns) > warpgate_count:
+                    player.warpgate_cooldowns.pop()
 
         elif self.type == 'NNet.Replay.Tracker.SUnitDoneEvent':
             obj.birth_time = gameloop
@@ -111,16 +149,25 @@ class ObjectEvent(BaseEvent):
             if 'worker' in obj.type:
                 summary_stats['workers_produced'][player.player_id] += 1
 
+            if not obj.position:
+                obj.position = {
+                    'x': event['m_x'],
+                    'y': event['m_y'],
+                }
+
         elif self.type == 'NNet.Replay.Tracker.SUnitDiedEvent':
             obj.status = 'died'
             obj.death_time = gameloop
 
+            if obj.name == 'WarpGate' and player.warpgate_cooldowns:
+                player.warpgate_cooldowns.pop()
+
             obj_killer_tag = event['m_killerUnitTagIndex']
             obj_killer_recycle = event['m_killerUnitTagRecycle']
 
             if obj_killer_tag and obj_killer_recycle:
                 obj_killer_id = protocol.unit_tag(obj_killer_tag, obj_killer_recycle)
-                for p in self.game.players:
+                for p in self.game.players.values():
                     if obj_killer_id in p.objects:
                         obj.killed_by = p.objects[obj_killer_id]
 
@@ -149,6 +196,9 @@ class ObjectEvent(BaseEvent):
             obj.gas_cost = new_obj_info['gas_cost']
             obj.supply = new_obj_info['supply']
 
+            if 'energy' in new_obj_info:
+                obj.energy = new_obj_info['energy']
+
             # organised in alphabetically sorted order
             morph_units = [
                 ['BanelingCocoon', 'Zergling'],
@@ -157,7 +207,8 @@ class ObjectEvent(BaseEvent):
                 ['OverlordTransport', 'TransportOverlordCocoon'],
                 ['Ravager', 'RavagerCocoon'],
                 ['LurkerMP', 'LurkerMPEgg'],
-                ['BroodLord', 'BroodLordCocoon']
+                ['BroodLord', 'BroodLordCocoon'],
+                ['CommandCenter', 'OrbitalCommand'],
             ]
 
             if sorted([old_name, new_obj_name]) in morph_units:
@@ -165,3 +216,5 @@ class ObjectEvent(BaseEvent):
 
             if 'Egg' in old_name or 'Cocoon' in old_name:
                 self._update_obj_group_info(obj)
+
+        # player.calc_supply()
